import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * MyWorld functions as the Controller for this Greenfoot
 * project which helps students understand inheritance
 * 
 * @Jordan Cohen && Brian Huang
 * @Feb 2012
 */
public class MyWorld extends World
{
    private int randomize;
    /**
     * spawnRate: Lower number means more spawns of cars
     * type: The type of Pedestrian being sent across
     * coolOff:A timer that limits the amount of Pedestrian being sent over at a time
     * lose: enable or disable to check if the player has lost
     * arrow: The object the player plays with
     * score: The score of the player
     * scoreCounter: The counter that shows the user their score
     * showType: The counter that shows what kind of Pedestrian is being sent over
     * backgroundMusic: The background Music
     */
    private int spawnRate;
    private int type;
    private int coolOff;
    private boolean lose;
    Actor arrow = new Arrow ();
    private int score;
    Counter scoreCounter = new Counter (8,"Score: ");
    Counter showType = new Counter (8, "Type: ");
    GreenfootSound backgroundMusic = new GreenfootSound ("background.mp3");

    private boolean hasScoreBoardAppeared;
    /**
     * Constructor for objects of class MyWorld.
     * NOTE: All the variables are reseted here because it allowes the reset button to work
     * super creates the world
     * All the variable are reseted for a clean game
     */
    public MyWorld()
    {    
        // Create a new world with 600x400 cells with a cell size of 1x1 pixels.
        super(600, 400, 1, false); 
        score = 0;
        lose = false;
        hasScoreBoardAppeared = false;
        type = 1;
        coolOff=0;
        spawnRate = 300;
        addObject(arrow, 300, 395);
        addObject (scoreCounter, 70, 20);
        showType.setValue (1);
        addObject (showType, 550, 20);
        scoreCounter.setValue (0);
    }

    /**
     * Method act
     * This is where the program loops and run the game
     */
    public void act ()
    {
        //checks if the game is over
        if (hasScoreBoardAppeared == false) {
            //play music
            backgroundMusic.playLoop ();
            //cecks for user input
            checkKeysAreDown ();

            //adds the scoreboard to the world if the player lost
            if (lose == true) {
                addScoreBoard ();
            }
            // Generate a random number to add a random element
            // to Vehicle spawning
            randomize = Greenfoot.getRandomNumber(spawnRate);

            // Chose a random lane in case a vehicle spawns
            int lane = Greenfoot.getRandomNumber (6);
            int spawnY = getYPosition (lane);

            // spawn vehicles
            //Their are 3 ways a car can be spawn so their are 3x more chance of a car being spawn
            if (randomize == 2 || randomize == 6 || randomize == 7)
            {
                // spawn a Car
                addObject (new Car(true), 10, spawnY);
            }
            else if (randomize == 3 )
            {
                // spawn a Bus
                addObject (new Bus(true), 10, spawnY);
            }

            else if (randomize == 4)
            {
                // spawn a Ambulance
                addObject (new Ambulance(true), 10, spawnY);
            }
            else if (randomize == 5)
            {
                // spawn a Plane
                addObject (new Plane(true), 10, spawnY);
            }

            //exactly the same as above, but these vehicles are spawn the oposite direction so the cars hit one another
            else if (randomize == 20 || randomize == 60 || randomize == 70)
            {
                // spawn a Car
                addObject (new Car(false), 600, spawnY);
            }
            else if (randomize == 30 )
            {
                // spawn a Bus
                addObject (new Bus(false), 600, spawnY);
            }

            else if (randomize == 40)
            {
                // spawn a Ambulance
                addObject (new Ambulance(false), 600, spawnY);
            }
            else if (randomize == 50)
            {
                // spawn a Plane
                addObject (new Plane(false), 600, spawnY);
            }
        }
    }

    /**
     * Method addPoints
     * A method for other classes to call to add points
     * By inserting an interger, this method adds points to the score
     */
    public void addPoints (int x) {
        //checks if the user lost first
        if (lose == false) {
            scoreCounter.add (x);
            score +=x;
        }
    }

    /**
     * Method lose
     * A method so other classes can call when the player has lost
     */
    public void lose () {
        lose = true;

    }

    /**
     * Returns the appropriate y coordinate for a given lane
     */
    private int getYPosition (int inLane)
    {
        // Manually input values based on the background graphic
        switch (inLane)
        {
            case 0: 
            return 79;

            case 1:
            return 127;

            case 2:
            return 175;

            case 3:
            return 222;

            case 4:
            return 272;

            case 5: 
            return 320;

        }  
        // In case an invalid value is passed in
        return 0;
    }

    /**
     * Method checkKeysAreDown
     * Check What keys are down, and depending on the action
     * Something happens.
     */
    private void checkKeysAreDown () {
        //If the player has the space key down
        //and the player has waited long enough
        //It spawns a new character
        if (Greenfoot.isKeyDown("space") && coolOff <= 0)
        {
            //if the type selected is 1, it spawns a runner,
            //If its 2, it spawns an Average Person
            //If its 3, it spawns a Little Boy
            //If its 4, it spawns a Baby
            if (type == 1) {
                addObject (new Runner (), arrow.getX (), 395);
            }
            else if (type == 2) {
                addObject (new AveragePerson (), arrow.getX (), 395);
            }
            else if (type == 3) {
                addObject (new LittleBoy (), arrow.getX (), 395);
            }
            else if (type == 4) {
                addObject (new Baby (), arrow.getX (), 395);
            }
            //Resets the coolOff Time
            coolOff = 30;
        }
        else {
            //If the player has not waited long enough, it's cooloff Time -1 
            coolOff --;
        }

        //Moves the Arrow Left or right
        if (Greenfoot.isKeyDown ("left")) {
            arrow.setLocation (arrow.getX () -2,arrow.getY ());
        }
        else if (Greenfoot.isKeyDown ("right")){
            arrow.setLocation (arrow.getX () +2,arrow.getY ());
        }
        //Checks if the player changes what type
        //Of pedestrian to send
        if (Greenfoot.isKeyDown ("1")) {
            type = 1;
            showType.setValue (1);
        }
        if (Greenfoot.isKeyDown ("2")) {
            type = 2;
            showType.setValue (2);
        }
        if (Greenfoot.isKeyDown ("3")) {
            type = 3;
            showType.setValue (3);
        }
        if (Greenfoot.isKeyDown ("4")) {
            type = 4;
            showType.setValue (4);
        }

    }

    /**
     * Method addScoreBoard
     * Adds the ScoreBoard with special messages and voice
     */
    private void addScoreBoard () {
        //stop the music
        backgroundMusic.stop ();
        
        if (score == 0) {
            ScoreBoard scoreBoard = new ScoreBoard ("Fantastic! You killed them all!", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("what-are-you-doing.wav").play ();

        }
        else if (score < 5) {
            ScoreBoard scoreBoard = new ScoreBoard ("Are you TRYING to kill them?", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("we-need-to-have.wav").play ();
            

        }
        else if (score < 10) {
            ScoreBoard scoreBoard = new ScoreBoard ("You seriously need glasses...", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;

            Greenfoot.delay (2);
            new GreenfootSound("what-are-you-doing.wav").play ();
        }
        else if (score < 15) {
            ScoreBoard scoreBoard = new ScoreBoard ("My granny could do better...", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("sigh-2.wav").play ();

        }
        else if (score < 20) {
            ScoreBoard scoreBoard = new ScoreBoard ("Not Bad for a n00b", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("sigh-2.wav").play ();

        }
        else if (score < 30) {
            ScoreBoard scoreBoard = new ScoreBoard ("Not Bad, could do better...", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("you-can-do-it.wav").play ();

        }
        else if (score < 30) {
            ScoreBoard scoreBoard = new ScoreBoard ("Try Again!", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("come-on-you-can-do-it.wav").play ();
        }
        else if (score < 40) {
            ScoreBoard scoreBoard = new ScoreBoard ("Almost had a good score", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("you-can-do-it.wav").play ();
        }
        else if (score < 50) {
            ScoreBoard scoreBoard = new ScoreBoard ("Good Job!", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("woohoo.wav").play ();
        }
        else if (score < 60) {
            ScoreBoard scoreBoard = new ScoreBoard ("Magnificent!", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("woohoo.wav").play ();
        }
        else if (score > 350) {
            ScoreBoard scoreBoard = new ScoreBoard ("You just hacked the game...RICKROLL! >.>", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("Never Gonna Give You Up.mp3").play ();
        }
        else {
            ScoreBoard scoreBoard = new ScoreBoard ("You deserve a medal!!", score);
            addObject (scoreBoard, 300, 200);
            hasScoreBoardAppeared = true;
            Greenfoot.delay (2);
            new GreenfootSound("unbelievable.wav").play ();
            
        }
        
    }
}

