import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Pedestrian Class
 * 
 * Moves mindlessly up the road, reacting to cars
 * that hit it, ambulances that heal it, and potentially
 * buses that pick it up.
 * 
 * @Jordan Cohen && Brian Huang
 * @Feburary 2012
 * 
 */

public class Pedestrian extends Actor
{
    /**
     * myWidth: Determines the width of the Pedestrian
     * 
     * startSpeed: Determines the speed of the Pedestrian 
     * depending on the type of Pedestrian
     * 
     * speed: The current speed of the Pedestrian
     * 
     * worth: The amount of points the Peedestrian is worth 
     * based on type
     * 
     * timeBeforeDeath: The player gets a bit more time after
     * the Pedestrian gets knocked over, and can only be saved
     * by:
     * A. A bus Touching it (send it automatically to the top)
     * B. A Ambulance (saves the pedestrian)
     * C. A plane (knocks it off the map)
     */
    private int myWidth;
    protected int startSpeed;
    protected int speed;
    protected int worth;
    private int timeBeforeDeath;

    /**
     * Method acting
     *
     *This method does not automatically run
     *(hence the name change from act --> acting)
     *This is so the specific Pedestrian that inherits
     *this class runs the same method but slightly different
     */
    public void acting() 
    {
        // figure out own width (related to checking if at world's edge)
        GreenfootImage g = this.getImage();
        myWidth = g.getWidth();

        // move upwards
        setLocation (getX(), getY() - speed);
        // check if I'm at the edge of the world,
        // and if so, remove myself
        if (atWorldEdge())
        {
            //If the pedestrian got to the finish line
            //The player gains point depending on the worth
            if (getY () <= 10) {
                MyWorld world = (MyWorld)getWorld();
                world.addPoints (worth);
                //Play a victory noise
                GreenfootSound victory = new GreenfootSound ("wow.wav");
                victory.play ();

            }
            getWorld().removeObject(this);
        }
        //The time before the pedestrian dies and the player loses
        if (getRotation () == 90) {
            timeBeforeDeath--;
            if (timeBeforeDeath == 1) {
                //If the Pedestrian is countdown to one
                //It sends a signal to the world that 
                //The player has lost
                MyWorld world = (MyWorld)getWorld();
                world.lose ();

                getWorld ().removeObject (this);

            }
        }
    }    

    /**
     * Method causes this Pedestrian to stop moving
     * and appear to fall over
     */
    public void knockMeOver ()
    {
        speed = 0;
        setRotation (90);
        timeBeforeDeath = 200;
        //Note: the sound crashes halfway while playing
        //so I did a catch to stop the program from crashing
        try {
            GreenfootSound scream = new GreenfootSound ("WilhelmScream.mp3");
            scream.play ();
        }
        catch (Exception e) {
        }
    }

    /**
     * Method causes this pedestrian to "heal" - regain
     * upright position and start moving again
     */
    public void healMe ()
    {
        speed = startSpeed;
        setRotation (0);
        timeBeforeDeath = 0;
        new GreenfootSound ("ambulance.wav").play ();

    }

    /**
     * Method causes the pedestrian to get picked up
     * by the bus and automatically gets to the finish line
     */
    public void pickMeUp()
    {
        this.setLocation (getX (),0);
        setRotation (0);
        timeBeforeDeath = 0;
        new GreenfootSound ("bus.mp3").play ();

    }

    /**
     * Method that causes the pedestrian to be pushed foward by the plane
     */
    public void pushed()
    {
        this.setLocation (getX () + 30,getY ());
        new GreenfootSound ("plane.wav").play ();
    }

    /**
     * Handy method that checks if this object is at the edge
     * of the World
     */
    public boolean atWorldEdge()
    {
        if (getX() < -(myWidth / 2) || getX() > getWorld().getWidth() + (myWidth / 2))
            return true;
        else if (getY() < -(myWidth / 2) || getY () > getWorld().getHeight() + (myWidth / 2))
            return true;
        else
            return false;
    }
}
